Package: httptest
Type: Package
Title: A Test Environment for HTTP Requests
Description: Testing and documenting code that communicates with remote servers
    can be painful. Dealing with authentication, server state,
    and other complications can make testing seem too costly to
    bother with. But it doesn't need to be that hard. This package enables one
    to test all of the logic on the R sides of the API in your package without
    requiring access to the remote service. Importantly, it provides three
    contexts that mock the network connection in different ways, as well as
    testing functions to assert that HTTP requests were---or were
    not---made. It also allows one to safely record real API responses to use as
    test fixtures. The ability to save responses and load them offline also
    enables one to write vignettes and other dynamic documents that can be
    distributed without access to a live server.
Version: 4.2.4
Authors@R: c(
        person("Neal", "Richardson", role=c("aut", "cre"), email="neal.p.richardson@gmail.com", comment=c(ORCID="0009-0002-7992-3520")),
        person("Jonathan", "Keane", role="ctb", email="jkeane@gmail.com"),
        person("Maëlle", "Salmon", role="ctb", email="maelle.salmon@yahoo.se", comment=c(ORCID="0000-0002-2815-0399"))
    )
URL: https://enpiar.com/r/httptest/,
        https://github.com/nealrichardson/httptest
BugReports: https://github.com/nealrichardson/httptest/issues
License: MIT + file LICENSE
Depends: R (>= 3.5.0), testthat
Imports: curl, digest, httr, jsonlite, stats, utils
Suggests: knitr, pkgload, rmarkdown, spelling, xml2
Language: en-US
RoxygenNote: 7.3.3
Encoding: UTF-8
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/Needs/coverage: covr
NeedsCompilation: no
Packaged: 2026-08-20 22:35:29 UTC; npr
Author: Neal Richardson [aut, cre] (ORCID:
    <https://orcid.org/0009-0002-7992-3520>),
  Jonathan Keane [ctb],
  Maëlle Salmon [ctb] (ORCID: <https://orcid.org/0000-0002-2815-0399>)
Maintainer: Neal Richardson <neal.p.richardson@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-21 12:50:18 UTC
