# Generated by roxygen2: do not edit by hand

export(.mockPaths)
export(block_requests)
export(build_mock_url)
export(capture_requests)
export(change_state)
export(end_vignette)
export(expect_DELETE)
export(expect_GET)
export(expect_PATCH)
export(expect_POST)
export(expect_PUT)
export(expect_header)
export(expect_json_equivalent)
export(expect_no_request)
export(fake_response)
export(find_mock_file)
export(get_current_redactor)
export(get_current_requester)
export(gsub_request)
export(gsub_response)
export(public)
export(redact_auth)
export(redact_cookies)
export(redact_headers)
export(save_response)
export(set_redactor)
export(set_requester)
export(skip_if_disconnected)
export(start_capturing)
export(start_vignette)
export(stop_capturing)
export(stop_mocking)
export(use_httptest)
export(use_mock_api)
export(with_fake_http)
export(with_mock_api)
export(with_mock_dir)
export(with_trace)
export(within_body_text)
export(without_internet)
importFrom(digest,digest)
importFrom(httr,DELETE)
importFrom(httr,GET)
importFrom(httr,PATCH)
importFrom(httr,POST)
importFrom(httr,PUT)
importFrom(httr,RETRY)
importFrom(httr,VERB)
importFrom(httr,add_headers)
importFrom(httr,content)
importFrom(jsonlite,prettify)
importFrom(jsonlite,toJSON)
importFrom(stats,terms)
importFrom(testthat,compare)
importFrom(testthat,expect)
importFrom(testthat,expect_error)
importFrom(testthat,expect_message)
importFrom(testthat,expect_warning)
importFrom(testthat,skip)
importFrom(testthat,skip_on_cran)
importFrom(utils,modifyList)
importFrom(utils,sessionInfo)
importFrom(utils,tail)
